==> reydb-1.0.6-py3-none-any.whl: reydb/rexecute.py <==
# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""
@Time    : 2022-12-05 14:10:02
@Author  : Rey
@Contact : reyxbo@163.com
@Explain : Database execute methods.
"""


from __future__ import annotations
from typing import Any, Self
from reykit.rexception import throw
from reykit.rtable import Table

from .rconnection import RDatabase, RDBConnection, RResult


__all__ = (
    'RDBExecute',
)


class RDBExecute(object):
    """
    Rey's `database execute` type.

    Examples
    --------
    Select.
    >>> field = ['id', 'value']
    >>> where = '`id` = ids'
    >>> ids = (1, 2)
    >>> result = RDBExecute.database.table(field, where, ids=ids)

    Insert.
    >>> data = [{'id': 1}, {'id': 2}]
    >>> duplicate = 'ignore'
    >>> result = RDBExecute.database.table + data
    >>> result = RDBExecute.database.table + (data, duplicate)
    >>> result = RDBExecute.database.table + {'data': data, 'duplicate': duplicate}

    Update.
    >>> data = [{'name': 'a', 'id': 1}, {'name': 'b', 'id': 2}]
    >>> where_fields = 'id'
    >>> result = RDBExecute.database.table & data
    >>> result = RDBExecute.database.table & (data, where_fields)
    >>> result = RDBExecute.database.table & {'data': data, 'where_fields': where_fields}

    Delete.
    >>> where = '`id` IN (1, 2)'
    >>> report = True
    >>> result = RDBExecute.database.table - where
    >>> result = RDBExecute.database.table - (where, report)
    >>> result = RDBExecute.database.table - {'where': where, 'report': report}

    Copy.
    >>> where = '`id` IN (1, 2)'
    >>> limit = 1
    >>> result = RDBExecute.database.table * where
    >>> result = RDBExecute.database.table * (where, limit)
    >>> result = RDBExecute.database.table * {'where': where, 'limit': limit}

    Exist.
    >>> where = '`id` IN (1, 2)'
    >>> report = True
    >>> result = where in RDBExecute.database.table
    >>> result = (where, report) in RDBExecute.database.table
    >>> result = {'where': where, 'report': report} in RDBExecute.database.table

    Count.
    >>> result = len(RDBExecute.database.table)

    Default database.
    >>> field = ['id', 'value']
    >>> engine = RDatabase(**server, database)
    >>> result = engine.exe.table()
    """


    def __init__(self, rdatabase: RDatabase | RDBConnection) -> None:
        """
        Build `database execute` attributes.

        Parameters
        ----------
        rdatabase : RDatabase or RDBConnection instance.
        """

        # Set parameter.
        self._rdatabase = rdatabase
        self._path: list[str] = []


    def __getattr__(self, name: str) -> Self:
        """
        Set database name or set table name.

        Parameters
        ----------
        name : Database name or table name.

        Returns
        -------
        Self.
        """

        # Check.
        if len(self._path) not in (0, 1):
            throw(AssertionError, self._path)

        # Set.
        self._path.append(name)

        return self


    @property
    def _get_path(self) -> tuple[str, str]:
        """
        Get database name and table name.

        Returns
        -------
        Database name and table name.
        """

        # Get.
        path_len = len(self._path)
        match path_len:
            case 1:
                database = self._rdatabase.database
                table = self._path[0]
            case 2:
                database = self._path[0]
                table = self._path[1]
            case _:
                throw(AssertionError)

        return database, table


    def __call__(
        self,
        *args: Any,
        **kwargs: Any
    ) -> RResult:
        """
        Select the data of table in the datebase.

        Parameters
        ----------
        args : Position arguments.
        kwargs : Keyword arguments.

        Returns
        -------
        Result object.
        """

        # Selete.
        result = self._rdatabase.execute_select(self._get_path, *args, **kwargs)

        return result


    def __add__(
        self,
        params: tuple | dict | Table
    ) -> RResult:
        """
        Insert the data of table in the datebase.

        Parameters
        ----------
        params : Insert parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `Table`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Insert.
        match params:
            case tuple():
                result = self._rdatabase.execute_insert(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_insert(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_insert(self._get_path, params)

        return result


    def __and__(
        self,
        params: tuple | dict | Table
    ) -> RResult:
        """
        Update the data of table in the datebase.

        Parameters
        ----------
        params : Update parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `Table`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Update.
        match params:
            case tuple():
                result = self._rdatabase.execute_update(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_update(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_update(self._get_path, params)

        return result


    def __sub__(
        self,
        params: tuple | dict | str
    ) -> RResult:
        """
        Delete the data of table in the datebase.

        Parameters
        ----------
        params : Update parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `str`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Update.
        match params:
            case tuple():
                result = self._rdatabase.execute_delete(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_delete(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_delete(self._get_path, params)

        return result


    def __mul__(
        self,
        params: tuple | dict | str
    ) -> RResult:
        """
        Copy record of table in the datebase.

        Parameters
        ----------
        params : Update parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `str`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Update.
        match params:
            case tuple():
                result = self._rdatabase.execute_copy(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_copy(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_copy(self._get_path, params)

        return result


    def __contains__(
        self,
        params: tuple | dict | str
    ) -> bool:
        """
        Judge the exist of record.

        Parameters
        ----------
        params : Update parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `str`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Update.
        match params:
            case tuple():
                result = self._rdatabase.execute_exist(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_exist(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_exist(self._get_path, params)

        return result


    def __len__(
        self
    ) -> int:
        """
        Count records.

        Returns
        -------
        Record count.
        """

        # Update.
        result = self._rdatabase.execute_count(self._get_path)

        return result

==> reydb-1.1.2-py3-none-any.whl: reydb/rexecute.py <==
# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""
@Time    : 2022-12-05 14:10:02
@Author  : Rey
@Contact : reyxbo@163.com
@Explain : Database execute methods.
"""


from __future__ import annotations
from typing import Any, Self
from reykit.rexception import throw
from reykit.rtable import Table
from reykit.rtype import RBase

from .rconnection import RDatabase, RDBConnection, RResult


__all__ = (
    'RDBExecute',
)


class RDBExecute(RBase):
    """
    Rey's `database execute` type.

    Examples
    --------
    Select.
    >>> field = ['id', 'value']
    >>> where = '`id` = ids'
    >>> ids = (1, 2)
    >>> result = RDBExecute.database.table(field, where, ids=ids)

    Insert.
    >>> data = [{'id': 1}, {'id': 2}]
    >>> duplicate = 'ignore'
    >>> result = RDBExecute.database.table + data
    >>> result = RDBExecute.database.table + (data, duplicate)
    >>> result = RDBExecute.database.table + {'data': data, 'duplicate': duplicate}

    Update.
    >>> data = [{'name': 'a', 'id': 1}, {'name': 'b', 'id': 2}]
    >>> where_fields = 'id'
    >>> result = RDBExecute.database.table & data
    >>> result = RDBExecute.database.table & (data, where_fields)
    >>> result = RDBExecute.database.table & {'data': data, 'where_fields': where_fields}

    Delete.
    >>> where = '`id` IN (1, 2)'
    >>> report = True
    >>> result = RDBExecute.database.table - where
    >>> result = RDBExecute.database.table - (where, report)
    >>> result = RDBExecute.database.table - {'where': where, 'report': report}

    Copy.
    >>> where = '`id` IN (1, 2)'
    >>> limit = 1
    >>> result = RDBExecute.database.table * where
    >>> result = RDBExecute.database.table * (where, limit)
    >>> result = RDBExecute.database.table * {'where': where, 'limit': limit}

    Exist.
    >>> where = '`id` IN (1, 2)'
    >>> report = True
    >>> result = where in RDBExecute.database.table
    >>> result = (where, report) in RDBExecute.database.table
    >>> result = {'where': where, 'report': report} in RDBExecute.database.table

    Count.
    >>> result = len(RDBExecute.database.table)

    Default database.
    >>> engine = RDatabase(**server, database)
    >>> result = engine.exe.table()
    """


    def __init__(self, rdatabase: RDatabase | RDBConnection) -> None:
        """
        Build `database execute` instance attributes.

        Parameters
        ----------
        rdatabase : RDatabase or RDBConnection instance.
        """

        # Set parameter.
        self._rdatabase = rdatabase
        self._path: list[str] = []


    def __getattr__(self, name: str) -> Self:
        """
        Set database name or set table name.

        Parameters
        ----------
        name : Database name or table name.

        Returns
        -------
        Self.
        """

        # Check.
        if len(self._path) not in (0, 1):
            throw(AssertionError, self._path)

        # Set.
        self._path.append(name)

        return self


    @property
    def _get_path(self) -> tuple[str, str]:
        """
        Get database name and table name.

        Returns
        -------
        Database name and table name.
        """

        # Get.
        path_len = len(self._path)
        match path_len:
            case 1:
                database = self._rdatabase.database
                table = self._path[0]
            case 2:
                database = self._path[0]
                table = self._path[1]
            case _:
                throw(AssertionError)

        return database, table


    def __call__(
        self,
        *args: Any,
        **kwargs: Any
    ) -> RResult:
        """
        Select the data of table in the datebase.

        Parameters
        ----------
        args : Position arguments.
        kwargs : Keyword arguments.

        Returns
        -------
        Result object.
        """

        # Selete.
        result = self._rdatabase.execute_select(self._get_path, *args, **kwargs)

        return result


    def __add__(
        self,
        params: tuple | dict | Table
    ) -> RResult:
        """
        Insert the data of table in the datebase.

        Parameters
        ----------
        params : Insert parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `Table`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Insert.
        match params:
            case tuple():
                result = self._rdatabase.execute_insert(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_insert(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_insert(self._get_path, params)

        return result


    def __and__(
        self,
        params: tuple | dict | Table
    ) -> RResult:
        """
        Update the data of table in the datebase.

        Parameters
        ----------
        params : Update parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `Table`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Update.
        match params:
            case tuple():
                result = self._rdatabase.execute_update(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_update(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_update(self._get_path, params)

        return result


    def __sub__(
        self,
        params: tuple | dict | str
    ) -> RResult:
        """
        Delete the data of table in the datebase.

        Parameters
        ----------
        params : Update parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `str`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Update.
        match params:
            case tuple():
                result = self._rdatabase.execute_delete(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_delete(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_delete(self._get_path, params)

        return result


    def __mul__(
        self,
        params: tuple | dict | str
    ) -> RResult:
        """
        Copy record of table in the datebase.

        Parameters
        ----------
        params : Update parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `str`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Update.
        match params:
            case tuple():
                result = self._rdatabase.execute_copy(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_copy(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_copy(self._get_path, params)

        return result


    def __contains__(
        self,
        params: tuple | dict | str
    ) -> bool:
        """
        Judge the exist of record.

        Parameters
        ----------
        params : Update parameters.
            - `tuple`: Enter parameters in '(path, *params)' format.
            - `dict`: Enter parameters in '(path, **params)' format.
            - `str`: Enter parameters in '(path, params)' format.

        Returns
        -------
        Result object.
        """

        # Update.
        match params:
            case tuple():
                result = self._rdatabase.execute_exist(self._get_path, *params)
            case dict():
                result = self._rdatabase.execute_exist(self._get_path, **params)
            case _:
                result = self._rdatabase.execute_exist(self._get_path, params)

        return result


    def __len__(
        self
    ) -> int:
        """
        Count records.

        Returns
        -------
        Record count.
        """

        # Update.
        result = self._rdatabase.execute_count(self._get_path)

        return result
